diff --git a/FL/Fl_Window.H b/FL/Fl_Window.H
index 03c8278..b63b875 100644
--- a/FL/Fl_Window.H
+++ b/FL/Fl_Window.H
@@ -450,6 +450,53 @@ public:
     \see virtual void Fl_Window::show()
   */
   void show(int argc, char **argv);
+
+  /**
+    Waits for the window to be fully displayed after calling show().
+
+    Fl_Window::show() is not guaranteed to show and draw the window on
+    all platforms immediately. Instead this is done in the background;
+    particularly on X11 this will take a few messages (client server
+    roundtrips) to display the window.
+
+    Currently (as of FLTK 1.3.3) this method only has an effect on X11,
+    as on Windows and Mac show() is already synchronous.
+
+    Usually the delay doesn't matter, but in some cases you may want to
+    have the window instantiated and displayed synchronously.
+
+    This method is intended to be used for displaying splash screens
+    before calling Fl::run() or having exact control over which window
+    has focus after calling show().
+
+    You can set \p blocking to true (1) to make this call block threads
+    that want to synchronize with the FLTK main loop. Otherwise
+    (\p blocking = 0, the default) threads that call Fl::lock() will not
+    be blocked while this method waits for the window to be displayed.
+
+    \note If the window is not shown(), the behavior is undefined.
+
+    \see virtual void Fl_Window::show()
+
+    Example code for displaying a window before calling Fl::run()
+
+      \code
+	Fl_Double_Window win = new Fl_Double_Window(...);
+
+	// do more window initialization here ...
+
+	win->show();			// show splash screen
+	win->wait_for_expose();		// wait, until displayed
+	Fl::flush();			// make sure everything gets drawn
+
+	// do more initialization work that needs some time ...
+
+	Fl::run();			// start FLTK event loop
+      \endcode
+  */
+
+  void wait_for_expose(int blocking=0);
+
   /**
     Makes the window completely fill one or more screens, without any
     window manager border visible.  You must use fullscreen_off() to
diff --git a/src/Fl_Window.cxx b/src/Fl_Window.cxx
index 6b8f167..9bd34a3 100644
--- a/src/Fl_Window.cxx
+++ b/src/Fl_Window.cxx
@@ -417,6 +417,16 @@ void Fl_Window::free_icons() {
 #endif
 }
 
+void Fl_Window::wait_for_expose(int blocking) {
+  if (!shown()) return;
+  if (blocking) Fl::lock();
+  while (!i || i->wait_for_expose) {
+    Fl::wait();
+  }
+  if (blocking) Fl::unlock();
+}
+
+
 //
 // End of "$Id$".
 //
